#!/usr/bin/env ruby
# by eto 2003-0112

require 'test/unit'
$LOAD_PATH << '../src'
require 'chise'
include CHISE

class TC_IDS < Test::Unit::TestCase
  def setup
  end
  def test_ids
    char = "榊".char
    assert_equal("⿰木神", char.ids)
    assert_equal("⿰木神", char.decompose)
    str = "榊"
    assert_equal("⿰木神", str.ids)
    assert_equal("⿰木神", str.decompose)
    assert_equal("⿰木⿰⺭申", str.decompose.decompose)
    assert_equal("⿰木神", str.decompose!)
    assert_equal("⿰木⿰⺭申", str.decompose!)
    str = "榊"
    assert_equal("⿰木⿰⺭申", str.decompose_all)
    assert_equal("⿰木⿰⺭申", str.decompose_all!)
    assert_equal("⿰木⿰⺭申", str)
    #今はまだcomposeはできない。

    de = "細".decompose
    assert_match(/田$/, de)  
    assert_equal(3, de.char_length)
    de = "&JX2-7577;".de_er.decompose
    de = "&CDP-8B60;".de_er.decompose
    assert_equal(1, de.char_length)
    de = "&JX2-217E;".de_er.decompose
    assert_match(/^⿰/, de)  
    assert_equal(3, de.char_length)
    assert_equal(6, de.decompose!.char_length)
#    assert_equal(6, de.decompose!.char_length)

    assert("⿸".char.is_ids?)
    assert(! "木".char.is_ids?)
    assert_equal(2, "⿰".char.ids_operator_argc)
    assert_equal(2, "&U+2FF0;".de_er.char.ids_operator_argc)
    assert_equal(2, "&U+2FF1;".de_er.char.ids_operator_argc)
    assert_equal(3, "&U+2FF2;".de_er.char.ids_operator_argc)
    assert_equal(3, "&U+2FF3;".de_er.char.ids_operator_argc)

    assert_equal("⿰", "&U+2FF0;".de_er.to_s)
    assert("&U+2FF0;".de_er.char.is_ids?)
    assert("&U+2FFF;".de_er.char.is_ids?)
    assert_match(/U\+2FF0/, "&U+2FF0;".de_er.char.inspect_x)
    assert_match(/IDEOGRAPHIC DESCRIPTION CHARACTER LEFT TO RIGHT/, "&U+2FF0;".de_er.char.inspect_all)
    (0x2FF0..0x2FFB).each {|i|
      assert_match(/IDEOGRAPHIC DESCRIPTION CHARACTER/, Character.new(i).name)
    }

    assert_match(/LEFT TO RIGHT/, "&U+2FF0;".de_er.name)		#∫
    assert_match(/ABOVE TO BELOW/, "&U+2FF1;".de_er.name)		#∨
    assert_match(/LEFT TO MIDDLE AND RIGHT/, "&U+2FF2;".de_er.name)	#∬
    assert_match(/ABOVE TO MIDDLE AND BELOW/, "&U+2FF3;".de_er.name)	#∀
    assert_match(/FULL SURROUND/, "&U+2FF4;".de_er.name)		#∃
    assert_match(/SURROUND FROM ABOVE/, "&U+2FF5;".de_er.name)		#∩
    assert_match(/SURROUND FROM BELOW/, "&U+2FF6;".de_er.name)		#∪
    assert_match(/SURROUND FROM LEFT/, "&U+2FF7;".de_er.name)		#⊂
    assert_match(/SURROUND FROM UPPER LEFT/, "&U+2FF8;".de_er.name)	#√
    assert_match(/SURROUND FROM UPPER RIGHT/, "&U+2FF9;".de_er.name)	#∂
    assert_match(/SURROUND FROM LOWER LEFT/, "&U+2FFA;".de_er.name)	#∠
    assert_match(/OVERLAID/, "&U+2FFB;".de_er.name) #∵
  end
  def test_tree
    assert_equal("[]", Tree.new().inspect)
    assert_equal("[1]", Tree.new().add_leaf(1).inspect)
    assert_equal("[1, 2]", Tree.new().add_leaf(1).add_leaf(2).inspect)
    assert_equal("[[]]", Tree.new().add_node.inspect)
    assert_equal("[[1]]", Tree.new().add_node.add_leaf(1).inspect)
    assert_equal("[[1, 2]]", Tree.new().add_node.add_leaf(1).add_leaf(2).inspect)
    assert_equal("[[1]]", Tree.new().add_node.add_leaf(1).end_node.inspect)
    assert_equal("[[1], [1]]", Tree.new().add_node.add_leaf(1).end_node.add_node.add_leaf(1).end_node.inspect)

    tree = Tree.new
    assert_equal("[]", tree.inspect)
    assert_equal("[1]", tree.add_leaf(1).inspect)
    assert_equal(0, tree.depth)
    assert_equal("[1, 2]", tree.add_leaf(2).inspect)
    assert_equal("[1, 2, []]", tree.add_node.inspect)
    assert_equal("[1, 2, [3]]", tree.add_leaf(3).inspect)
    assert_equal(1, tree.depth)
    assert_equal("[1, 2, [3, 4]]", tree.add_leaf(4).inspect)
    assert_equal("[1, 2, [3, 4]]", tree.end_node.inspect)
    assert_equal("[1, 2, [3, 4], [5]]", tree.add_node.add_leaf(5).inspect)
    assert_equal("[1, 2, [3, 4], [5, [6]]]", tree.add_node.add_leaf(6).inspect)
    assert_equal(2, tree.depth)

    tree = Tree.new
    assert_equal('[["+"]]', tree.add_node("+", 2).inspect)
    assert_equal('[["+", 1]]', tree.add_leaf(1).inspect)
    assert_equal("unmatch leaves", tree.check_integrity)
    assert_equal('[["+", 1, 2]]', tree.add_leaf(2).inspect)
    assert_nil(tree.check_integrity)
    assert_equal('[["+", 1, 2], 3]', tree.add_leaf(3).inspect)
    assert_equal("extra nodes", tree.check_integrity)
    
    tree = Tree.new
    assert_equal('[["+"]]', tree.add_node("+", 2).inspect)
    assert_equal("unmatch leaves", tree.check_integrity)
    assert_equal('[["+", 1]]', tree.add_leaf(1).inspect)
    assert_equal("unmatch leaves", tree.check_integrity)
    assert_equal('[["+", 1, ["+"]]]', tree.add_node("+", 2).inspect)
    assert_equal("unmatch leaves", tree.check_integrity)
    assert_equal('[["+", 1, ["+", 2]]]', tree.add_leaf(2).inspect)
    assert_equal("unmatch leaves", tree.check_integrity)
    assert_equal('[["+", 1, ["+", 2, 3]]]', tree.add_leaf(3).inspect)
    assert_nil(tree.check_integrity)

    tree = Tree.new
    assert_equal('[1]', tree.add_leaf(1).inspect)
    assert_nil(tree.check_integrity)
    assert_equal('[1, 2]', tree.add_leaf(2).inspect)
    assert_equal("extra leaves", tree.check_integrity)
  end
  def test_ids_tree
    assert_equal('[[<+,U+002B>, <A,U+0041>, <B,U+0042>]]', IDS_Tree.new("+AB").inspect)
    assert_equal('[[<+,U+002B>, <A,U+0041>, <B,U+0042>], <C,U+0043>]', IDS_Tree.new("+ABC").inspect)
    assert_equal('[[<+,U+002B>, <A,U+0041>, [<+,U+002B>, <B,U+0042>, <C,U+0043>]]]', IDS_Tree.new("+A+BC").inspect)
    assert_equal('[[<+,U+002B>, <A,U+0041>, [<+,U+002B>, <B,U+0042>, <C,U+0043>]], <D,U+0044>]', IDS_Tree.new("+A+BCD").inspect)

    assert_equal('[<榊,U+698A>]', IDS_Tree.new("榊").inspect)
#    assert_equal('[[<⿰,U+2FF0>, <木,J90-4C5A>, <神,J90-3F40>]]', IDS_Tree.new("⿰木神").inspect)
    assert_equal(1, IDS_Tree.new("⿰木神").depth)
#    assert_equal('[[<⿰,U+2FF0>, <木,J90-4C5A>, [<⿰,U+2FF0>, <⺭,CDP-8B70>, <申,J90-3F3D>]]]', IDS_Tree.new("⿰木⿰⺭申").inspect)
    assert_equal(2, IDS_Tree.new("⿰木⿰⺭申").depth)
    assert_equal("unmatch leaves", IDS_Tree.new("⿰木").check_integrity)
    assert_nil(IDS_Tree.new("⿰木神").check_integrity)
    assert_equal("unmatch leaves", IDS_Tree.new("⿰木⿰申").check_integrity)
    assert_nil(IDS_Tree.new("⿰木⿰⺭申").check_integrity)
    assert_equal("extra nodes", IDS_Tree.new("⿰木⿰⺭申申").check_integrity)
    assert_nil(IDS_Tree.new("榊").check_integrity)
    assert_equal("extra leaves", IDS_Tree.new("榊榊").check_integrity)

    assert_equal(3, "⿳".char.ids_operator_argc)
    assert_equal("⿳士冖匕", "壱".char.ids)
    assert_equal(3, "壱".char.ids.char.ids_operator_argc)
    assert_nil(IDS_Tree.new("⿳士冖匕").check_integrity)
    assert_equal("unmatch leaves", IDS_Tree.new("⿳士冖").check_integrity)
    assert_equal("extra nodes", IDS_Tree.new("⿳士冖匕匕").check_integrity)

    assert_equal("contains ques", IDS_Tree.new("⿳士冖?").check_integrity)
  end
  def test_tree_depth
    assert_equal(1, IDS_Tree.new("林".decompose).depth)
    assert_equal('["⿰木木"]', IDS_Tree.new("林".decompose).nodes.inspect)
    assert_equal('[]', IDS_Tree.new("林".decompose).sub_nodes.inspect)
    assert_equal(2, IDS_Tree.new("榊".decompose_all).depth)
    assert_equal('["⿰木⿰⺭申", "⿰⺭申"]', IDS_Tree.new("榊".decompose_all).nodes.inspect)
    assert_equal('["⿰⺭申"]', IDS_Tree.new("榊".decompose_all).sub_nodes.inspect)

    assert_equal(3, IDS_Tree.new("焔".decompose_all).depth)
    assert_equal(3, IDS_Tree.new("焔".decompose_all).nodes.length)
    assert_equal(2, IDS_Tree.new("焔".decompose_all).sub_nodes.length)

    assert_equal(2, IDS_Tree.new("屡".decompose_all).depth)
    assert_equal("⿸尸娄", "⿸尸⿱米女".aggregate)
    assert_equal(3, IDS_Tree.new("醤".decompose_all).depth)
  end
  def test_compose_exact #正確に一致するIDSを検知する
    assert_equal("榊", "榊".decompose.compose)
    assert_equal("壱", "壱".decompose.compose)
    assert_equal("⿰木木", "林".decompose)
    assert_equal("林", "⿰木木".compose)
    assert_equal("林", "林".decompose.compose)
    assert_equal("⿰木木", "⿰木木".compose.decompose)
    assert_equal("林".ucs, "⿰木木".compose.ucs)
  end
  def test_find
#    p "日雲".find #"曇"
    assert_equal(4, "日雲".find .char_length) #"曇"
  end
  def test_compose_part
#    p de.compose_ar
#    p "神".compose_ar
#    p "木".compose_ar.join
  end
  def test_aggregate
#    db = IDS_DB.instance
#    db.list_aggregate
  end
  def test_ids_error
#    p "実".char.inspect_all
#    p "実".char.ids
#    assert_equal("contains ques", "実".char.ids_error)
    assert_equal("unmatch leaves", "実".char.ids_error)
#    p CharDB.instance.get('ascii').keys
#    p CharDB.instance.get('no-such-attribute').keys
#    p CharDB.instance.get('ids-error').keys
  end
end

#----------------------------------------------------------------------end.
